# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.pagination import AsyncPager, SyncPager
from ..core.request_options import RequestOptions
from ..types.connection_for_organization import ConnectionForOrganization
from ..types.create_organization_response_content import CreateOrganizationResponseContent
from ..types.create_token_quota import CreateTokenQuota
from ..types.get_organization_by_name_response_content import GetOrganizationByNameResponseContent
from ..types.get_organization_response_content import GetOrganizationResponseContent
from ..types.list_organizations_paginated_response_content import ListOrganizationsPaginatedResponseContent
from ..types.organization import Organization
from ..types.organization_branding import OrganizationBranding
from ..types.organization_metadata import OrganizationMetadata
from ..types.organization_third_party_client_access_enum import OrganizationThirdPartyClientAccessEnum
from ..types.update_organization_response_content import UpdateOrganizationResponseContent
from ..types.update_token_quota import UpdateTokenQuota
from .raw_client import AsyncRawOrganizationsClient, RawOrganizationsClient

if typing.TYPE_CHECKING:
    from .client_grants.client import AsyncClientGrantsClient, ClientGrantsClient
    from .connections.client import AsyncConnectionsClient, ConnectionsClient
    from .discovery_domains.client import AsyncDiscoveryDomainsClient, DiscoveryDomainsClient
    from .enabled_connections.client import AsyncEnabledConnectionsClient, EnabledConnectionsClient
    from .groups.client import AsyncGroupsClient, GroupsClient
    from .invitations.client import AsyncInvitationsClient, InvitationsClient
    from .members.client import AsyncMembersClient, MembersClient
    from .roles.client import AsyncRolesClient, RolesClient
# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class OrganizationsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawOrganizationsClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._client_grants: typing.Optional[ClientGrantsClient] = None
        self._connections: typing.Optional[ConnectionsClient] = None
        self._discovery_domains: typing.Optional[DiscoveryDomainsClient] = None
        self._enabled_connections: typing.Optional[EnabledConnectionsClient] = None
        self._invitations: typing.Optional[InvitationsClient] = None
        self._members: typing.Optional[MembersClient] = None
        self._groups: typing.Optional[GroupsClient] = None
        self._roles: typing.Optional[RolesClient] = None

    @property
    def with_raw_response(self) -> RawOrganizationsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawOrganizationsClient
        """
        return self._raw_client

    def list(
        self,
        *,
        from_: typing.Optional[str] = None,
        take: typing.Optional[int] = 50,
        sort: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[Organization, ListOrganizationsPaginatedResponseContent]:
        """
        Retrieve detailed list of all Organizations available in your tenant. For more information, see Auth0 Organizations.

        This endpoint supports two types of pagination:

        - Offset pagination
        - Checkpoint pagination

        Checkpoint pagination must be used if you need to retrieve more than 1000 organizations.

        **Checkpoint Pagination**

        To search by checkpoint, use the following parameters:

        - `from`: Optional id from which to start selection.
        - `take`: The total number of entries to retrieve when using the `from` parameter. Defaults to 50.

        **Note**: The first time you call this endpoint using checkpoint pagination, omit the `from` parameter. If there are more results, a `next` value is included in the response. You can use this for subsequent API calls. When `next` is no longer included in the response, no pages are remaining.

        Parameters
        ----------
        from_ : typing.Optional[str]
            Optional Id from which to start selection.

        take : typing.Optional[int]
            Number of results per page. Defaults to 50.

        sort : typing.Optional[str]
            Field to sort by. Use <code>field:order</code> where order is <code>1</code> for ascending and <code>-1</code> for descending. e.g. <code>created_at:1</code>. We currently support sorting by the following fields: <code>name</code>, <code>display_name</code> and <code>created_at</code>.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[Organization, ListOrganizationsPaginatedResponseContent]
            Organizations successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        response = client.organizations.list(
            from_="from",
            take=1,
            sort="sort",
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        return self._raw_client.list(from_=from_, take=take, sort=sort, request_options=request_options)

    def create(
        self,
        *,
        name: str,
        display_name: typing.Optional[str] = OMIT,
        branding: typing.Optional[OrganizationBranding] = OMIT,
        metadata: typing.Optional[OrganizationMetadata] = OMIT,
        enabled_connections: typing.Optional[typing.Sequence[ConnectionForOrganization]] = OMIT,
        token_quota: typing.Optional[CreateTokenQuota] = OMIT,
        third_party_client_access: typing.Optional[OrganizationThirdPartyClientAccessEnum] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateOrganizationResponseContent:
        """
        Create a new Organization within your tenant.  To learn more about Organization settings, behavior, and configuration options, review [Create Your First Organization](https://auth0.com/docs/manage-users/organizations/create-first-organization).

        Parameters
        ----------
        name : str
            The name of this organization.

        display_name : typing.Optional[str]
            Friendly name of this organization.

        branding : typing.Optional[OrganizationBranding]

        metadata : typing.Optional[OrganizationMetadata]

        enabled_connections : typing.Optional[typing.Sequence[ConnectionForOrganization]]
            Connections that will be enabled for this organization. See POST enabled_connections endpoint for the object format. (Max of 10 connections allowed)

        token_quota : typing.Optional[CreateTokenQuota]

        third_party_client_access : typing.Optional[OrganizationThirdPartyClientAccessEnum]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateOrganizationResponseContent
            Organization successfully created.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.organizations.create(
            name="name",
        )
        """
        _response = self._raw_client.create(
            name=name,
            display_name=display_name,
            branding=branding,
            metadata=metadata,
            enabled_connections=enabled_connections,
            token_quota=token_quota,
            third_party_client_access=third_party_client_access,
            request_options=request_options,
        )
        return _response.data

    def get_by_name(
        self, name: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetOrganizationByNameResponseContent:
        """
        Retrieve details about a single Organization specified by name.

        Parameters
        ----------
        name : str
            name of the organization to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetOrganizationByNameResponseContent
            Organization successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.organizations.get_by_name(
            name="name",
        )
        """
        _response = self._raw_client.get_by_name(name, request_options=request_options)
        return _response.data

    def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetOrganizationResponseContent:
        """
        Retrieve details about a single Organization specified by ID.

        Parameters
        ----------
        id : str
            ID of the organization to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetOrganizationResponseContent
            Organization successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.organizations.get(
            id="id",
        )
        """
        _response = self._raw_client.get(id, request_options=request_options)
        return _response.data

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Remove an Organization from your tenant.  This action cannot be undone.

        **Note**: Members are automatically disassociated from an Organization when it is deleted. However, this action does **not** delete these users from your tenant.

        Parameters
        ----------
        id : str
            Organization identifier.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.organizations.delete(
            id="id",
        )
        """
        _response = self._raw_client.delete(id, request_options=request_options)
        return _response.data

    def update(
        self,
        id: str,
        *,
        display_name: typing.Optional[str] = OMIT,
        name: typing.Optional[str] = OMIT,
        branding: typing.Optional[OrganizationBranding] = OMIT,
        metadata: typing.Optional[OrganizationMetadata] = OMIT,
        token_quota: typing.Optional[UpdateTokenQuota] = OMIT,
        third_party_client_access: typing.Optional[OrganizationThirdPartyClientAccessEnum] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateOrganizationResponseContent:
        """
        Update the details of a specific [Organization](https://auth0.com/docs/manage-users/organizations/configure-organizations/create-organizations), such as name and display name, branding options, and metadata.

        Parameters
        ----------
        id : str
            ID of the organization to update.

        display_name : typing.Optional[str]
            Friendly name of this organization.

        name : typing.Optional[str]
            The name of this organization.

        branding : typing.Optional[OrganizationBranding]

        metadata : typing.Optional[OrganizationMetadata]

        token_quota : typing.Optional[UpdateTokenQuota]

        third_party_client_access : typing.Optional[OrganizationThirdPartyClientAccessEnum]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateOrganizationResponseContent
            Organization successfully updated.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.organizations.update(
            id="id",
        )
        """
        _response = self._raw_client.update(
            id,
            display_name=display_name,
            name=name,
            branding=branding,
            metadata=metadata,
            token_quota=token_quota,
            third_party_client_access=third_party_client_access,
            request_options=request_options,
        )
        return _response.data

    @property
    def client_grants(self):
        if self._client_grants is None:
            from .client_grants.client import ClientGrantsClient  # noqa: E402

            self._client_grants = ClientGrantsClient(client_wrapper=self._client_wrapper)
        return self._client_grants

    @property
    def connections(self):
        if self._connections is None:
            from .connections.client import ConnectionsClient  # noqa: E402

            self._connections = ConnectionsClient(client_wrapper=self._client_wrapper)
        return self._connections

    @property
    def discovery_domains(self):
        if self._discovery_domains is None:
            from .discovery_domains.client import DiscoveryDomainsClient  # noqa: E402

            self._discovery_domains = DiscoveryDomainsClient(client_wrapper=self._client_wrapper)
        return self._discovery_domains

    @property
    def enabled_connections(self):
        if self._enabled_connections is None:
            from .enabled_connections.client import EnabledConnectionsClient  # noqa: E402

            self._enabled_connections = EnabledConnectionsClient(client_wrapper=self._client_wrapper)
        return self._enabled_connections

    @property
    def invitations(self):
        if self._invitations is None:
            from .invitations.client import InvitationsClient  # noqa: E402

            self._invitations = InvitationsClient(client_wrapper=self._client_wrapper)
        return self._invitations

    @property
    def members(self):
        if self._members is None:
            from .members.client import MembersClient  # noqa: E402

            self._members = MembersClient(client_wrapper=self._client_wrapper)
        return self._members

    @property
    def groups(self):
        if self._groups is None:
            from .groups.client import GroupsClient  # noqa: E402

            self._groups = GroupsClient(client_wrapper=self._client_wrapper)
        return self._groups

    @property
    def roles(self):
        if self._roles is None:
            from .roles.client import RolesClient  # noqa: E402

            self._roles = RolesClient(client_wrapper=self._client_wrapper)
        return self._roles


class AsyncOrganizationsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawOrganizationsClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._client_grants: typing.Optional[AsyncClientGrantsClient] = None
        self._connections: typing.Optional[AsyncConnectionsClient] = None
        self._discovery_domains: typing.Optional[AsyncDiscoveryDomainsClient] = None
        self._enabled_connections: typing.Optional[AsyncEnabledConnectionsClient] = None
        self._invitations: typing.Optional[AsyncInvitationsClient] = None
        self._members: typing.Optional[AsyncMembersClient] = None
        self._groups: typing.Optional[AsyncGroupsClient] = None
        self._roles: typing.Optional[AsyncRolesClient] = None

    @property
    def with_raw_response(self) -> AsyncRawOrganizationsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawOrganizationsClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        from_: typing.Optional[str] = None,
        take: typing.Optional[int] = 50,
        sort: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[Organization, ListOrganizationsPaginatedResponseContent]:
        """
        Retrieve detailed list of all Organizations available in your tenant. For more information, see Auth0 Organizations.

        This endpoint supports two types of pagination:

        - Offset pagination
        - Checkpoint pagination

        Checkpoint pagination must be used if you need to retrieve more than 1000 organizations.

        **Checkpoint Pagination**

        To search by checkpoint, use the following parameters:

        - `from`: Optional id from which to start selection.
        - `take`: The total number of entries to retrieve when using the `from` parameter. Defaults to 50.

        **Note**: The first time you call this endpoint using checkpoint pagination, omit the `from` parameter. If there are more results, a `next` value is included in the response. You can use this for subsequent API calls. When `next` is no longer included in the response, no pages are remaining.

        Parameters
        ----------
        from_ : typing.Optional[str]
            Optional Id from which to start selection.

        take : typing.Optional[int]
            Number of results per page. Defaults to 50.

        sort : typing.Optional[str]
            Field to sort by. Use <code>field:order</code> where order is <code>1</code> for ascending and <code>-1</code> for descending. e.g. <code>created_at:1</code>. We currently support sorting by the following fields: <code>name</code>, <code>display_name</code> and <code>created_at</code>.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[Organization, ListOrganizationsPaginatedResponseContent]
            Organizations successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.organizations.list(
                from_="from",
                take=1,
                sort="sort",
            )
            async for item in response:
                yield item

            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        return await self._raw_client.list(from_=from_, take=take, sort=sort, request_options=request_options)

    async def create(
        self,
        *,
        name: str,
        display_name: typing.Optional[str] = OMIT,
        branding: typing.Optional[OrganizationBranding] = OMIT,
        metadata: typing.Optional[OrganizationMetadata] = OMIT,
        enabled_connections: typing.Optional[typing.Sequence[ConnectionForOrganization]] = OMIT,
        token_quota: typing.Optional[CreateTokenQuota] = OMIT,
        third_party_client_access: typing.Optional[OrganizationThirdPartyClientAccessEnum] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateOrganizationResponseContent:
        """
        Create a new Organization within your tenant.  To learn more about Organization settings, behavior, and configuration options, review [Create Your First Organization](https://auth0.com/docs/manage-users/organizations/create-first-organization).

        Parameters
        ----------
        name : str
            The name of this organization.

        display_name : typing.Optional[str]
            Friendly name of this organization.

        branding : typing.Optional[OrganizationBranding]

        metadata : typing.Optional[OrganizationMetadata]

        enabled_connections : typing.Optional[typing.Sequence[ConnectionForOrganization]]
            Connections that will be enabled for this organization. See POST enabled_connections endpoint for the object format. (Max of 10 connections allowed)

        token_quota : typing.Optional[CreateTokenQuota]

        third_party_client_access : typing.Optional[OrganizationThirdPartyClientAccessEnum]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateOrganizationResponseContent
            Organization successfully created.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.organizations.create(
                name="name",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            name=name,
            display_name=display_name,
            branding=branding,
            metadata=metadata,
            enabled_connections=enabled_connections,
            token_quota=token_quota,
            third_party_client_access=third_party_client_access,
            request_options=request_options,
        )
        return _response.data

    async def get_by_name(
        self, name: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetOrganizationByNameResponseContent:
        """
        Retrieve details about a single Organization specified by name.

        Parameters
        ----------
        name : str
            name of the organization to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetOrganizationByNameResponseContent
            Organization successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.organizations.get_by_name(
                name="name",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_by_name(name, request_options=request_options)
        return _response.data

    async def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetOrganizationResponseContent:
        """
        Retrieve details about a single Organization specified by ID.

        Parameters
        ----------
        id : str
            ID of the organization to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetOrganizationResponseContent
            Organization successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.organizations.get(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(id, request_options=request_options)
        return _response.data

    async def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Remove an Organization from your tenant.  This action cannot be undone.

        **Note**: Members are automatically disassociated from an Organization when it is deleted. However, this action does **not** delete these users from your tenant.

        Parameters
        ----------
        id : str
            Organization identifier.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.organizations.delete(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(id, request_options=request_options)
        return _response.data

    async def update(
        self,
        id: str,
        *,
        display_name: typing.Optional[str] = OMIT,
        name: typing.Optional[str] = OMIT,
        branding: typing.Optional[OrganizationBranding] = OMIT,
        metadata: typing.Optional[OrganizationMetadata] = OMIT,
        token_quota: typing.Optional[UpdateTokenQuota] = OMIT,
        third_party_client_access: typing.Optional[OrganizationThirdPartyClientAccessEnum] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateOrganizationResponseContent:
        """
        Update the details of a specific [Organization](https://auth0.com/docs/manage-users/organizations/configure-organizations/create-organizations), such as name and display name, branding options, and metadata.

        Parameters
        ----------
        id : str
            ID of the organization to update.

        display_name : typing.Optional[str]
            Friendly name of this organization.

        name : typing.Optional[str]
            The name of this organization.

        branding : typing.Optional[OrganizationBranding]

        metadata : typing.Optional[OrganizationMetadata]

        token_quota : typing.Optional[UpdateTokenQuota]

        third_party_client_access : typing.Optional[OrganizationThirdPartyClientAccessEnum]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateOrganizationResponseContent
            Organization successfully updated.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.organizations.update(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(
            id,
            display_name=display_name,
            name=name,
            branding=branding,
            metadata=metadata,
            token_quota=token_quota,
            third_party_client_access=third_party_client_access,
            request_options=request_options,
        )
        return _response.data

    @property
    def client_grants(self):
        if self._client_grants is None:
            from .client_grants.client import AsyncClientGrantsClient  # noqa: E402

            self._client_grants = AsyncClientGrantsClient(client_wrapper=self._client_wrapper)
        return self._client_grants

    @property
    def connections(self):
        if self._connections is None:
            from .connections.client import AsyncConnectionsClient  # noqa: E402

            self._connections = AsyncConnectionsClient(client_wrapper=self._client_wrapper)
        return self._connections

    @property
    def discovery_domains(self):
        if self._discovery_domains is None:
            from .discovery_domains.client import AsyncDiscoveryDomainsClient  # noqa: E402

            self._discovery_domains = AsyncDiscoveryDomainsClient(client_wrapper=self._client_wrapper)
        return self._discovery_domains

    @property
    def enabled_connections(self):
        if self._enabled_connections is None:
            from .enabled_connections.client import AsyncEnabledConnectionsClient  # noqa: E402

            self._enabled_connections = AsyncEnabledConnectionsClient(client_wrapper=self._client_wrapper)
        return self._enabled_connections

    @property
    def invitations(self):
        if self._invitations is None:
            from .invitations.client import AsyncInvitationsClient  # noqa: E402

            self._invitations = AsyncInvitationsClient(client_wrapper=self._client_wrapper)
        return self._invitations

    @property
    def members(self):
        if self._members is None:
            from .members.client import AsyncMembersClient  # noqa: E402

            self._members = AsyncMembersClient(client_wrapper=self._client_wrapper)
        return self._members

    @property
    def groups(self):
        if self._groups is None:
            from .groups.client import AsyncGroupsClient  # noqa: E402

            self._groups = AsyncGroupsClient(client_wrapper=self._client_wrapper)
        return self._groups

    @property
    def roles(self):
        if self._roles is None:
            from .roles.client import AsyncRolesClient  # noqa: E402

            self._roles = AsyncRolesClient(client_wrapper=self._client_wrapper)
        return self._roles



# This file was auto-generated by Fern from our API Definition.

from auth0.management.core.query_encoder import encode_query


def test_query_encoding_deep_objects() -> None:
    assert encode_query({"hello world": "hello world"}) == [("hello world", "hello world")]
    assert encode_query({"hello_world": {"hello": "world"}}) == [("hello_world[hello]", "world")]
    assert encode_query({"hello_world": {"hello": {"world": "today"}, "test": "this"}, "hi": "there"}) == [
        ("hello_world[hello][world]", "today"),
        ("hello_world[test]", "this"),
        ("hi", "there"),
    ]


def test_query_encoding_deep_object_arrays() -> None:
    assert encode_query({"objects": [{"key": "hello", "value": "world"}, {"key": "foo", "value": "bar"}]}) == [
        ("objects[key]", "hello"),
        ("objects[value]", "world"),
        ("objects[key]", "foo"),
        ("objects[value]", "bar"),
    ]
    assert encode_query(
        {"users": [{"name": "string", "tags": ["string"]}, {"name": "string2", "tags": ["string2", "string3"]}]}
    ) == [
        ("users[name]", "string"),
        ("users[tags]", "string"),
        ("users[name]", "string2"),
        ("users[tags]", "string2"),
        ("users[tags]", "string3"),
    ]


def test_encode_query_with_none() -> None:
    encoded = encode_query(None)
    assert encoded is None



# This file was auto-generated by Fern from our API Definition.

ConnectionRequiresUsername = bool
"""
Indicates whether the user is required to provide a username in addition to an email address.
"""



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .event_stream_cloud_event_group_role_deleted_object_group import EventStreamCloudEventGroupRoleDeletedObjectGroup
from .event_stream_cloud_event_group_role_deleted_object_role import EventStreamCloudEventGroupRoleDeletedObjectRole


class EventStreamCloudEventGroupRoleDeletedObject(UniversalBaseModel):
    """
    The event content.
    """

    group: EventStreamCloudEventGroupRoleDeletedObjectGroup
    role: EventStreamCloudEventGroupRoleDeletedObjectRole
    deleted_at: dt.datetime = pydantic.Field()
    """
    The time at which the role was removed from the group.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ....core.api_error import ApiError
from ....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ....core.http_response import AsyncHttpResponse, HttpResponse
from ....core.jsonable_encoder import encode_path_param
from ....core.pagination import AsyncPager, SyncPager
from ....core.parse_error import ParsingError
from ....core.pydantic_utilities import parse_obj_as
from ....core.request_options import RequestOptions
from ....errors.bad_request_error import BadRequestError
from ....errors.conflict_error import ConflictError
from ....errors.forbidden_error import ForbiddenError
from ....errors.not_found_error import NotFoundError
from ....errors.precondition_failed_error import PreconditionFailedError
from ....errors.too_many_requests_error import TooManyRequestsError
from ....errors.unauthorized_error import UnauthorizedError
from ....types.action_module_version import ActionModuleVersion
from ....types.create_action_module_version_response_content import CreateActionModuleVersionResponseContent
from ....types.get_action_module_version_response_content import GetActionModuleVersionResponseContent
from ....types.get_action_module_versions_response_content import GetActionModuleVersionsResponseContent
from pydantic import ValidationError


class RawVersionsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self,
        id: str,
        *,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[ActionModuleVersion, GetActionModuleVersionsResponseContent]:
        """
        List all published versions of a specific Actions Module.

        Parameters
        ----------
        id : str
            The unique ID of the module.

        page : typing.Optional[int]
            Use this field to request a specific page of the list results.

        per_page : typing.Optional[int]
            The maximum number of results to be returned by the server in a single response. 20 by default.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[ActionModuleVersion, GetActionModuleVersionsResponseContent]
            The module versions were retrieved.
        """
        page = page if page is not None else 0

        _response = self._client_wrapper.httpx_client.request(
            f"actions/modules/{encode_path_param(id)}/versions",
            method="GET",
            params={
                "page": page,
                "per_page": per_page,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    GetActionModuleVersionsResponseContent,
                    parse_obj_as(
                        type_=GetActionModuleVersionsResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.versions
                _has_next = len(_items or []) > 0
                _get_next = lambda: self.list(
                    id,
                    page=page + 1,
                    per_page=per_page,
                    request_options=request_options,
                )
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next, response=_parsed_response)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[CreateActionModuleVersionResponseContent]:
        """
        Creates a new immutable version of an Actions Module from the current draft version. This publishes the draft as a new version that can be referenced by actions, while maintaining the existing draft for continued development.

        Parameters
        ----------
        id : str
            The ID of the action module to create a version for.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreateActionModuleVersionResponseContent]
            The action module version was created.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"actions/modules/{encode_path_param(id)}/versions",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateActionModuleVersionResponseContent,
                    parse_obj_as(
                        type_=CreateActionModuleVersionResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 412:
                raise PreconditionFailedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get(
        self, id: str, version_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetActionModuleVersionResponseContent]:
        """
        Retrieve the details of a specific, immutable version of an Actions Module.

        Parameters
        ----------
        id : str
            The unique ID of the module.

        version_id : str
            The unique ID of the module version to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetActionModuleVersionResponseContent]
            The module version was retrieved.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"actions/modules/{encode_path_param(id)}/versions/{encode_path_param(version_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetActionModuleVersionResponseContent,
                    parse_obj_as(
                        type_=GetActionModuleVersionResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawVersionsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self,
        id: str,
        *,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[ActionModuleVersion, GetActionModuleVersionsResponseContent]:
        """
        List all published versions of a specific Actions Module.

        Parameters
        ----------
        id : str
            The unique ID of the module.

        page : typing.Optional[int]
            Use this field to request a specific page of the list results.

        per_page : typing.Optional[int]
            The maximum number of results to be returned by the server in a single response. 20 by default.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[ActionModuleVersion, GetActionModuleVersionsResponseContent]
            The module versions were retrieved.
        """
        page = page if page is not None else 0

        _response = await self._client_wrapper.httpx_client.request(
            f"actions/modules/{encode_path_param(id)}/versions",
            method="GET",
            params={
                "page": page,
                "per_page": per_page,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    GetActionModuleVersionsResponseContent,
                    parse_obj_as(
                        type_=GetActionModuleVersionsResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.versions
                _has_next = len(_items or []) > 0

                async def _get_next():
                    return await self.list(
                        id,
                        page=page + 1,
                        per_page=per_page,
                        request_options=request_options,
                    )

                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next, response=_parsed_response)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[CreateActionModuleVersionResponseContent]:
        """
        Creates a new immutable version of an Actions Module from the current draft version. This publishes the draft as a new version that can be referenced by actions, while maintaining the existing draft for continued development.

        Parameters
        ----------
        id : str
            The ID of the action module to create a version for.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreateActionModuleVersionResponseContent]
            The action module version was created.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"actions/modules/{encode_path_param(id)}/versions",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateActionModuleVersionResponseContent,
                    parse_obj_as(
                        type_=CreateActionModuleVersionResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 412:
                raise PreconditionFailedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get(
        self, id: str, version_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetActionModuleVersionResponseContent]:
        """
        Retrieve the details of a specific, immutable version of an Actions Module.

        Parameters
        ----------
        id : str
            The unique ID of the module.

        version_id : str
            The unique ID of the module version to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetActionModuleVersionResponseContent]
            The module version was retrieved.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"actions/modules/{encode_path_param(id)}/versions/{encode_path_param(version_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetActionModuleVersionResponseContent,
                    parse_obj_as(
                        type_=GetActionModuleVersionResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .cimd_mapped_client_fields import CimdMappedClientFields
from .cimd_validation_result import CimdValidationResult


class PreviewCimdMetadataResponseContent(UniversalBaseModel):
    client_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The client_id of an existing client registered with this external_client_id, if one exists.
    """

    errors: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Array of retrieval errors (populated when the metadata document could not be fetched). When present, validation is omitted.
    """

    validation: typing.Optional[CimdValidationResult] = None
    mapped_fields: typing.Optional[CimdMappedClientFields] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



from .conftest import get_client, verify_request_count


def test_actions_triggers_bindings_list_() -> None:
    """Test list endpoint with WireMock"""
    test_id = "actions.triggers.bindings.list_.0"
    client = get_client(test_id)
    client.actions.triggers.bindings.list(
        trigger_id="post-login",
        page=1,
        per_page=1,
    )
    verify_request_count(test_id, "GET", "/actions/triggers/post-login/bindings", {"page": "1", "per_page": "1"}, 1)


def test_actions_triggers_bindings_update_many() -> None:
    """Test updateMany endpoint with WireMock"""
    test_id = "actions.triggers.bindings.update_many.0"
    client = get_client(test_id)
    client.actions.triggers.bindings.update_many(
        trigger_id="post-login",
    )
    verify_request_count(test_id, "PATCH", "/actions/triggers/post-login/bindings", None, 1)



# This file was auto-generated by Fern from our API Definition.

import typing

EventStreamCloudEventConnectionCreatedObject0OptionsAttributeMapMappingModeEnum = typing.Union[
    typing.Literal["bind_all", "use_map"], typing.Any
]



from .conftest import get_client, verify_request_count


def test_actions_executions_get() -> None:
    """Test get endpoint with WireMock"""
    test_id = "actions.executions.get.0"
    client = get_client(test_id)
    client.actions.executions.get(
        id="id",
    )
    verify_request_count(test_id, "GET", "/actions/executions/id", None, 1)



# This file was auto-generated by Fern from our API Definition.

import typing

from .flow_action_hubspot_enroll_contact import FlowActionHubspotEnrollContact
from .flow_action_hubspot_get_contact import FlowActionHubspotGetContact
from .flow_action_hubspot_upsert_contact import FlowActionHubspotUpsertContact

FlowActionHubspot = typing.Union[
    FlowActionHubspotEnrollContact, FlowActionHubspotGetContact, FlowActionHubspotUpsertContact
]



# This file was auto-generated by Fern from our API Definition.

import typing

GuardianFactorNameEnum = typing.Union[
    typing.Literal[
        "push-notification", "sms", "email", "duo", "otp", "webauthn-roaming", "webauthn-platform", "recovery-code"
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.connection_key import ConnectionKey
from ...types.post_connection_keys_request_content import PostConnectionKeysRequestContent
from ...types.post_connections_keys_response_content import PostConnectionsKeysResponseContent
from ...types.rotate_connection_keys_request_content import RotateConnectionKeysRequestContent
from ...types.rotate_connections_keys_response_content import RotateConnectionsKeysResponseContent
from .raw_client import AsyncRawKeysClient, RawKeysClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class KeysClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawKeysClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawKeysClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawKeysClient
        """
        return self._raw_client

    def get(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> typing.List[ConnectionKey]:
        """
        Gets the connection keys for the Okta or OIDC connection strategy.

        Parameters
        ----------
        id : str
            ID of the connection

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[ConnectionKey]
            Connection keys successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.connections.keys.get(
            id="id",
        )
        """
        _response = self._raw_client.get(id, request_options=request_options)
        return _response.data

    def create(
        self,
        id: str,
        *,
        request: typing.Optional[PostConnectionKeysRequestContent] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PostConnectionsKeysResponseContent:
        """
        Provision initial connection keys for Okta or OIDC connection strategies. This endpoint allows you to create keys before configuring the connection to use Private Key JWT authentication, enabling zero-downtime transitions.

        Parameters
        ----------
        id : str
            ID of the connection

        request : typing.Optional[PostConnectionKeysRequestContent]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PostConnectionsKeysResponseContent
            Connection keys successfully created.

        Examples
        --------
        from auth0 import Auth0, PostConnectionKeysRequestContent

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.connections.keys.create(
            id="id",
            request=PostConnectionKeysRequestContent(),
        )
        """
        _response = self._raw_client.create(id, request=request, request_options=request_options)
        return _response.data

    def rotate(
        self,
        id: str,
        *,
        request: typing.Optional[RotateConnectionKeysRequestContent] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> RotateConnectionsKeysResponseContent:
        """
        Rotates the connection keys for the Okta or OIDC connection strategies.

        Parameters
        ----------
        id : str
            ID of the connection

        request : typing.Optional[RotateConnectionKeysRequestContent]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RotateConnectionsKeysResponseContent
            Connection keys successfully rotated.

        Examples
        --------
        from auth0 import Auth0, RotateConnectionKeysRequestContent

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.connections.keys.rotate(
            id="id",
            request=RotateConnectionKeysRequestContent(),
        )
        """
        _response = self._raw_client.rotate(id, request=request, request_options=request_options)
        return _response.data


class AsyncKeysClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawKeysClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawKeysClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawKeysClient
        """
        return self._raw_client

    async def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[ConnectionKey]:
        """
        Gets the connection keys for the Okta or OIDC connection strategy.

        Parameters
        ----------
        id : str
            ID of the connection

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[ConnectionKey]
            Connection keys successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.connections.keys.get(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(id, request_options=request_options)
        return _response.data

    async def create(
        self,
        id: str,
        *,
        request: typing.Optional[PostConnectionKeysRequestContent] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PostConnectionsKeysResponseContent:
        """
        Provision initial connection keys for Okta or OIDC connection strategies. This endpoint allows you to create keys before configuring the connection to use Private Key JWT authentication, enabling zero-downtime transitions.

        Parameters
        ----------
        id : str
            ID of the connection

        request : typing.Optional[PostConnectionKeysRequestContent]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PostConnectionsKeysResponseContent
            Connection keys successfully created.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0, PostConnectionKeysRequestContent

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.connections.keys.create(
                id="id",
                request=PostConnectionKeysRequestContent(),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(id, request=request, request_options=request_options)
        return _response.data

    async def rotate(
        self,
        id: str,
        *,
        request: typing.Optional[RotateConnectionKeysRequestContent] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> RotateConnectionsKeysResponseContent:
        """
        Rotates the connection keys for the Okta or OIDC connection strategies.

        Parameters
        ----------
        id : str
            ID of the connection

        request : typing.Optional[RotateConnectionKeysRequestContent]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RotateConnectionsKeysResponseContent
            Connection keys successfully rotated.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0, RotateConnectionKeysRequestContent

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.connections.keys.rotate(
                id="id",
                request=RotateConnectionKeysRequestContent(),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.rotate(id, request=request, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

from .x_509_certificate_credential import X509CertificateCredential

CreateClientAuthenticationMethodSelfSignedTlsClientAuthCredentials = typing.List[X509CertificateCredential]
"""
Fully defined credentials that will be enabled on the client for mTLS authentication utilizing self-signed certificate.
"""




import unittest
from unittest import mock
import json

import requests
from auth0.authentication.exceptions import Auth0Error, RateLimitError

from auth0.authentication.back_channel_login import BackChannelLogin

class TestBackChannelLogin(unittest.TestCase):
    @mock.patch("auth0.authentication.rest.RestClient.post")
    def test_ciba(self, mock_post):
        g = BackChannelLogin("my.domain.com", "cid", client_secret="clsec")

        g.back_channel_login(
            binding_message="This is a binding message",
            login_hint="{ \"format\": \"iss_sub\", \"iss\": \"https://my.domain.auth0.com/\", \"sub\": \"auth0|[USER ID]\" }",
            scope="openid",
        )

        args, kwargs = mock_post.call_args

        self.assertEqual(args[0], "https://my.domain.com/bc-authorize")
        self.assertEqual(
            kwargs["data"],
            {
                "client_id": "cid",
                "client_secret": "clsec",
                "binding_message": "This is a binding message",
                "login_hint": "{ \"format\": \"iss_sub\", \"iss\": \"https://my.domain.auth0.com/\", \"sub\": \"auth0|[USER ID]\" }",
                "scope": "openid",
            },
        )

    @mock.patch("requests.request")
    def test_server_error(self, mock_requests_request):
        response = requests.Response()
        response.status_code = 400
        response._content = b'{"error":"foo"}'
        mock_requests_request.return_value = response

        g = BackChannelLogin("my.domain.com", "cid", client_secret="clsec")
        with self.assertRaises(Auth0Error) as context:
            g.back_channel_login(
                binding_message="msg",
                login_hint="hint",
                scope="openid"
            )
        self.assertEqual(context.exception.status_code, 400)
        self.assertEqual(context.exception.message, 'foo')

    @mock.patch("auth0.authentication.rest.RestClient.post")
    def test_should_require_binding_message(self, mock_post):
        g = BackChannelLogin("my.domain.com", "cid", client_secret="clsec")

        # Expecting an exception to be raised when binding_message is missing
        with self.assertRaises(Exception) as context:
            g.back_channel_login(
                login_hint='{ "format": "iss_sub", "iss": "https://my.domain.auth0.com/", "sub": "auth0|USER_ID" }',
                scope="openid",
            )

        # Assert the error message is correct
        self.assertIn("missing 1 required positional argument: \'binding_message\'", str(context.exception))

    @mock.patch("auth0.authentication.rest.RestClient.post")
    def test_should_require_login_hint(self, mock_post):
        g = BackChannelLogin("my.domain.com", "cid", client_secret="clsec")

        # Expecting an exception to be raised when login_hint is missing
        with self.assertRaises(Exception) as context:
            g.back_channel_login(
                binding_message="This is a binding message.",
                scope="openid",
            )

        # Assert the error message is correct
        self.assertIn("missing 1 required positional argument: \'login_hint\'", str(context.exception))
    
    @mock.patch("auth0.authentication.rest.RestClient.post")
    def test_should_require_scope(self, mock_post):
        g = BackChannelLogin("my.domain.com", "cid", client_secret="clsec")

        # Expecting an exception to be raised when scope is missing
        with self.assertRaises(Exception) as context:
            g.back_channel_login(
                binding_message="This is a binding message.",
                login_hint='{ "format": "iss_sub", "iss": "https://my.domain.auth0.com/", "sub": "auth0|USER_ID" }',
            )

        # Assert the error message is correct
        self.assertIn("missing 1 required positional argument: \'scope\'", str(context.exception))

    @mock.patch("auth0.authentication.rest.RestClient.post")
    def test_with_authorization_details(self, mock_post):
        g = BackChannelLogin("my.domain.com", "cid", client_secret="clsec")
        g.back_channel_login(
            binding_message="This is a binding message.",
            login_hint= json.dumps({"format": "iss_sub", "iss": "https://my.domain.auth0.com/", "sub": "auth0|USER_ID"}),
            scope="openid",
            authorization_details=[
                {
                    "type":"payment_initiation","locations":["https://example.com/payments"],
                    "instructedAmount":
                    {
                        "currency":"EUR","amount":"123.50"
                    },
                    "creditorName":"Merchant A",
                    "creditorAccount":
                    {
                        "bic":"ABCIDEFFXXX",
                        "iban":"DE021001001093071118603"
                    },
                    "remittanceInformationUnstructured":"Ref Number Merchant"
                }
            ],
        )

        args, kwargs = mock_post.call_args

        expected_data = {
            "client_id": "cid",
            "client_secret": "clsec",
            "binding_message": "This is a binding message.",
            "login_hint": json.dumps({"format": "iss_sub", "iss": "https://my.domain.auth0.com/", "sub": "auth0|USER_ID"}),
            "scope": "openid",
            "authorization_details": json.dumps([
                {
                    "type":"payment_initiation","locations":["https://example.com/payments"],
                    "instructedAmount":
                    {
                        "currency":"EUR","amount":"123.50"
                    },
                    "creditorName":"Merchant A",
                    "creditorAccount":
                    {
                        "bic":"ABCIDEFFXXX",
                        "iban":"DE021001001093071118603"
                    },
                    "remittanceInformationUnstructured":"Ref Number Merchant"
                }
            ]),
        }

        actual_data = kwargs["data"]

        self.assertEqual(args[0], "https://my.domain.com/bc-authorize")

        self.assertEqual(
            actual_data,
            expected_data,
            "Request data does not match expected data after JSON serialization."
        )

    @mock.patch("auth0.authentication.rest.RestClient.post")
    def test_with_request_expiry(self, mock_post):
        g = BackChannelLogin("my.domain.com", "cid", client_secret="clsec")

        g.back_channel_login(
            binding_message="This is a binding message",
            login_hint="{ \"format\": \"iss_sub\", \"iss\": \"https://my.domain.auth0.com/\", \"sub\": \"auth0|[USER ID]\" }",
            scope="openid",
            requested_expiry=100
        )

        args, kwargs = mock_post.call_args

        self.assertEqual(args[0], "https://my.domain.com/bc-authorize")
        self.assertEqual(
            kwargs["data"],
            {
                "client_id": "cid",
                "client_secret": "clsec",
                "binding_message": "This is a binding message",
                "login_hint": "{ \"format\": \"iss_sub\", \"iss\": \"https://my.domain.auth0.com/\", \"sub\": \"auth0|[USER ID]\" }",
                "scope": "openid",
                "requested_expiry": "100",
            },
        )

    def test_requested_expiry_negative_raises(self):
        g = BackChannelLogin("my.domain.com", "cid", client_secret="clsec")
        with self.assertRaises(ValueError):
            g.back_channel_login(
                binding_message="msg",
                login_hint="hint",
                scope="openid",
                requested_expiry=-10
            )

    def test_requested_expiry_zero_raises(self):
        g = BackChannelLogin("my.domain.com", "cid", client_secret="clsec")
        with self.assertRaises(ValueError):
            g.back_channel_login(
                binding_message="msg",
                login_hint="hint",
                scope="openid",
                requested_expiry=0
            )

    def test_requested_non_int_raises(self):
        g = BackChannelLogin("my.domain.com", "cid", client_secret="clsec")
        with self.assertRaises(ValueError):
            g.back_channel_login(
                binding_message="msg",
                login_hint="hint",
                scope="openid",
                requested_expiry="string_instead_of_int"
            )



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .created_user_authentication_method_type_enum import CreatedUserAuthenticationMethodTypeEnum
from .credential_device_type_enum import CredentialDeviceTypeEnum
from .preferred_authentication_method_enum import PreferredAuthenticationMethodEnum
from .user_authentication_method_properties import UserAuthenticationMethodProperties


class CreateUserAuthenticationMethodResponseContent(UniversalBaseModel):
    """
    The successfully created authentication method.
    """

    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the newly created authentication method (automatically generated by the application)
    """

    type: CreatedUserAuthenticationMethodTypeEnum
    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    A human-readable label to identify the authentication method.
    """

    totp_secret: typing.Optional[str] = pydantic.Field(default=None)
    """
    Base32 encoded secret for TOTP generation
    """

    phone_number: typing.Optional[str] = pydantic.Field(default=None)
    """
    Applies to phone authentication methods only. The destination phone number used to send verification codes via text and voice.
    """

    email: typing.Optional[str] = pydantic.Field(default=None)
    """
    Applies to email authentication methods only. The email address used to send verification messages.
    """

    authentication_methods: typing.Optional[typing.List[UserAuthenticationMethodProperties]] = None
    preferred_authentication_method: typing.Optional[PreferredAuthenticationMethodEnum] = pydantic.Field(default=None)
    """
    Preferred phone authentication method
    """

    key_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Applies to webauthn authenticators only. The id of the credential.
    """

    public_key: typing.Optional[str] = pydantic.Field(default=None)
    """
    Applies to webauthn authenticators only. The public key.
    """

    aaguid: typing.Optional[str] = pydantic.Field(default=None)
    """
    Applies to passkeys only. Authenticator Attestation Globally Unique Identifier.
    """

    relying_party_identifier: typing.Optional[str] = pydantic.Field(default=None)
    """
    Applies to webauthn authenticators only. The relying party identifier.
    """

    credential_device_type: typing.Optional[CredentialDeviceTypeEnum] = None
    credential_backed_up: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Applies to passkeys only. Whether the credential was backed up.
    """

    identity_user_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Applies to passkeys only. The ID of the user identity linked with the authentication method.
    """

    user_agent: typing.Optional[str] = pydantic.Field(default=None)
    """
    Applies to passkeys only. The user-agent of the browser used to create the passkey.
    """

    user_handle: typing.Optional[str] = pydantic.Field(default=None)
    """
    Applies to passkeys only. The user handle of the user identity.
    """

    transports: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Applies to passkeys only. The transports used by clients to communicate with the authenticator.
    """

    created_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    Authentication method creation date
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.brute_force_protection_mode_enum import BruteForceProtectionModeEnum
from ...types.brute_force_protection_shields_enum import BruteForceProtectionShieldsEnum
from ...types.get_brute_force_settings_response_content import GetBruteForceSettingsResponseContent
from ...types.update_brute_force_settings_response_content import UpdateBruteForceSettingsResponseContent
from .raw_client import AsyncRawBruteForceProtectionClient, RawBruteForceProtectionClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class BruteForceProtectionClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawBruteForceProtectionClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawBruteForceProtectionClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawBruteForceProtectionClient
        """
        return self._raw_client

    def get(self, *, request_options: typing.Optional[RequestOptions] = None) -> GetBruteForceSettingsResponseContent:
        """
        Retrieve details of the Brute-force Protection configuration of your tenant.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetBruteForceSettingsResponseContent
            Brute force configuration successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.attack_protection.brute_force_protection.get()
        """
        _response = self._raw_client.get(request_options=request_options)
        return _response.data

    def update(
        self,
        *,
        enabled: typing.Optional[bool] = OMIT,
        shields: typing.Optional[typing.Sequence[BruteForceProtectionShieldsEnum]] = OMIT,
        allowlist: typing.Optional[typing.Sequence[str]] = OMIT,
        mode: typing.Optional[BruteForceProtectionModeEnum] = OMIT,
        max_attempts: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateBruteForceSettingsResponseContent:
        """
        Update the Brute-force Protection configuration of your tenant.

        Parameters
        ----------
        enabled : typing.Optional[bool]
            Whether or not brute force attack protections are active.

        shields : typing.Optional[typing.Sequence[BruteForceProtectionShieldsEnum]]
            Action to take when a brute force protection threshold is violated.
                    Possible values: <code>block</code>, <code>user_notification</code>.

        allowlist : typing.Optional[typing.Sequence[str]]
            List of trusted IP addresses that will not have attack protection enforced against them.

        mode : typing.Optional[BruteForceProtectionModeEnum]

        max_attempts : typing.Optional[int]
            Maximum number of unsuccessful attempts.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateBruteForceSettingsResponseContent
            Brute force configuration successfully updated.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.attack_protection.brute_force_protection.update()
        """
        _response = self._raw_client.update(
            enabled=enabled,
            shields=shields,
            allowlist=allowlist,
            mode=mode,
            max_attempts=max_attempts,
            request_options=request_options,
        )
        return _response.data


class AsyncBruteForceProtectionClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawBruteForceProtectionClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawBruteForceProtectionClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawBruteForceProtectionClient
        """
        return self._raw_client

    async def get(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetBruteForceSettingsResponseContent:
        """
        Retrieve details of the Brute-force Protection configuration of your tenant.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetBruteForceSettingsResponseContent
            Brute force configuration successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.attack_protection.brute_force_protection.get()


        asyncio.run(main())
        """
        _response = await self._raw_client.get(request_options=request_options)
        return _response.data

    async def update(
        self,
        *,
        enabled: typing.Optional[bool] = OMIT,
        shields: typing.Optional[typing.Sequence[BruteForceProtectionShieldsEnum]] = OMIT,
        allowlist: typing.Optional[typing.Sequence[str]] = OMIT,
        mode: typing.Optional[BruteForceProtectionModeEnum] = OMIT,
        max_attempts: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateBruteForceSettingsResponseContent:
        """
        Update the Brute-force Protection configuration of your tenant.

        Parameters
        ----------
        enabled : typing.Optional[bool]
            Whether or not brute force attack protections are active.

        shields : typing.Optional[typing.Sequence[BruteForceProtectionShieldsEnum]]
            Action to take when a brute force protection threshold is violated.
                    Possible values: <code>block</code>, <code>user_notification</code>.

        allowlist : typing.Optional[typing.Sequence[str]]
            List of trusted IP addresses that will not have attack protection enforced against them.

        mode : typing.Optional[BruteForceProtectionModeEnum]

        max_attempts : typing.Optional[int]
            Maximum number of unsuccessful attempts.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateBruteForceSettingsResponseContent
            Brute force configuration successfully updated.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.attack_protection.brute_force_protection.update()


        asyncio.run(main())
        """
        _response = await self._raw_client.update(
            enabled=enabled,
            shields=shields,
            allowlist=allowlist,
            mode=mode,
            max_attempts=max_attempts,
            request_options=request_options,
        )
        return _response.data



from __future__ import annotations

from typing import Any

from .base import AuthenticationBase


class Passwordless(AuthenticationBase):

    """Passwordless connections endpoints.

    Args:
        domain (str): Your auth0 domain (e.g: my-domain.us.auth0.com)
    """

    def email(
        self, email: str, send: str = "link", auth_params: dict[str, str] | None = None
    ) -> Any:
        """Start flow sending an email.

        Given the user email address, it will send an email with:

          - A link (default, send:"link"). You can then authenticate with
            this user opening the link and he will be automatically logged in
            to the application. Optionally, you can append/override
            parameters to the link (like scope, redirect_uri, protocol,
            response_type, etc.) using auth_params dict.

          - A verification code (send:"code"). You can then authenticate with
            this user using email as username and code as password.

        Complete the authentication using the get_token.passwordless_login method.

        Args:
            email (str): Email address.

            send (str, optional): Can be: 'link' or 'code'. Defaults to 'link'.

            auth_params (dict, optional): Parameters to append or override.
        """

        data: dict[str, Any] = {
            "client_id": self.client_id,
            "connection": "email",
            "email": email,
            "send": send,
        }
        if auth_params:
            data.update({"authParams": auth_params})

        return self.authenticated_post(
            f"{self.protocol}://{self.domain}/passwordless/start", data=data
        )

    def sms(self, phone_number: str) -> Any:
        """Start flow sending an SMS message.

        Given the user phone number, it will send an SMS with
        a verification code. You can then authenticate with
        this user using phone number as username and code as password.

        Complete the authentication using the get_token.passwordless_login method.

        Args:
            phone_number (str): Phone number.
        """

        data = {
            "client_id": self.client_id,
            "connection": "sms",
            "phone_number": phone_number,
        }

        return self.authenticated_post(
            f"{self.protocol}://{self.domain}/passwordless/start", data=data
        )



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .user_attribute_profile_id import UserAttributeProfileId
from .user_attribute_profile_name import UserAttributeProfileName
from .user_attribute_profile_user_attributes import UserAttributeProfileUserAttributes
from .user_attribute_profile_user_id import UserAttributeProfileUserId


class GetUserAttributeProfileResponseContent(UniversalBaseModel):
    id: typing.Optional[UserAttributeProfileId] = None
    name: typing.Optional[UserAttributeProfileName] = None
    user_id: typing.Optional[UserAttributeProfileUserId] = None
    user_attributes: typing.Optional[UserAttributeProfileUserAttributes] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ....core.pagination import AsyncPager, SyncPager
from ....core.request_options import RequestOptions
from ....types.action_module_version import ActionModuleVersion
from ....types.create_action_module_version_response_content import CreateActionModuleVersionResponseContent
from ....types.get_action_module_version_response_content import GetActionModuleVersionResponseContent
from ....types.get_action_module_versions_response_content import GetActionModuleVersionsResponseContent
from .raw_client import AsyncRawVersionsClient, RawVersionsClient


class VersionsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawVersionsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawVersionsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawVersionsClient
        """
        return self._raw_client

    def list(
        self,
        id: str,
        *,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[ActionModuleVersion, GetActionModuleVersionsResponseContent]:
        """
        List all published versions of a specific Actions Module.

        Parameters
        ----------
        id : str
            The unique ID of the module.

        page : typing.Optional[int]
            Use this field to request a specific page of the list results.

        per_page : typing.Optional[int]
            The maximum number of results to be returned by the server in a single response. 20 by default.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[ActionModuleVersion, GetActionModuleVersionsResponseContent]
            The module versions were retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        response = client.actions.modules.versions.list(
            id="id",
            page=1,
            per_page=1,
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        return self._raw_client.list(id, page=page, per_page=per_page, request_options=request_options)

    def create(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> CreateActionModuleVersionResponseContent:
        """
        Creates a new immutable version of an Actions Module from the current draft version. This publishes the draft as a new version that can be referenced by actions, while maintaining the existing draft for continued development.

        Parameters
        ----------
        id : str
            The ID of the action module to create a version for.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateActionModuleVersionResponseContent
            The action module version was created.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.actions.modules.versions.create(
            id="id",
        )
        """
        _response = self._raw_client.create(id, request_options=request_options)
        return _response.data

    def get(
        self, id: str, version_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetActionModuleVersionResponseContent:
        """
        Retrieve the details of a specific, immutable version of an Actions Module.

        Parameters
        ----------
        id : str
            The unique ID of the module.

        version_id : str
            The unique ID of the module version to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetActionModuleVersionResponseContent
            The module version was retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.actions.modules.versions.get(
            id="id",
            version_id="versionId",
        )
        """
        _response = self._raw_client.get(id, version_id, request_options=request_options)
        return _response.data


class AsyncVersionsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawVersionsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawVersionsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawVersionsClient
        """
        return self._raw_client

    async def list(
        self,
        id: str,
        *,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[ActionModuleVersion, GetActionModuleVersionsResponseContent]:
        """
        List all published versions of a specific Actions Module.

        Parameters
        ----------
        id : str
            The unique ID of the module.

        page : typing.Optional[int]
            Use this field to request a specific page of the list results.

        per_page : typing.Optional[int]
            The maximum number of results to be returned by the server in a single response. 20 by default.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[ActionModuleVersion, GetActionModuleVersionsResponseContent]
            The module versions were retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.actions.modules.versions.list(
                id="id",
                page=1,
                per_page=1,
            )
            async for item in response:
                yield item

            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        return await self._raw_client.list(id, page=page, per_page=per_page, request_options=request_options)

    async def create(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> CreateActionModuleVersionResponseContent:
        """
        Creates a new immutable version of an Actions Module from the current draft version. This publishes the draft as a new version that can be referenced by actions, while maintaining the existing draft for continued development.

        Parameters
        ----------
        id : str
            The ID of the action module to create a version for.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateActionModuleVersionResponseContent
            The action module version was created.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.actions.modules.versions.create(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(id, request_options=request_options)
        return _response.data

    async def get(
        self, id: str, version_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetActionModuleVersionResponseContent:
        """
        Retrieve the details of a specific, immutable version of an Actions Module.

        Parameters
        ----------
        id : str
            The unique ID of the module.

        version_id : str
            The unique ID of the module version to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetActionModuleVersionResponseContent
            The module version was retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.actions.modules.versions.get(
                id="id",
                version_id="versionId",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(id, version_id, request_options=request_options)
        return _response.data



from .conftest import get_client, verify_request_count


def test_forms_list_() -> None:
    """Test list endpoint with WireMock"""
    test_id = "forms.list_.0"
    client = get_client(test_id)
    client.forms.list(
        page=1,
        per_page=1,
        include_totals=True,
        hydrate=["flow_count"],
    )
    verify_request_count(
        test_id, "GET", "/forms", {"page": "1", "per_page": "1", "include_totals": "true", "hydrate": "flow_count"}, 1
    )


def test_forms_create() -> None:
    """Test create endpoint with WireMock"""
    test_id = "forms.create.0"
    client = get_client(test_id)
    client.forms.create(
        name="name",
    )
    verify_request_count(test_id, "POST", "/forms", None, 1)


def test_forms_get() -> None:
    """Test get endpoint with WireMock"""
    test_id = "forms.get.0"
    client = get_client(test_id)
    client.forms.get(
        id="id",
        hydrate=["flow_count"],
    )
    verify_request_count(test_id, "GET", "/forms/id", {"hydrate": "flow_count"}, 1)


def test_forms_delete() -> None:
    """Test delete endpoint with WireMock"""
    test_id = "forms.delete.0"
    client = get_client(test_id)
    client.forms.delete(
        id="id",
    )
    verify_request_count(test_id, "DELETE", "/forms/id", None, 1)


def test_forms_update() -> None:
    """Test update endpoint with WireMock"""
    test_id = "forms.update.0"
    client = get_client(test_id)
    client.forms.update(
        id="id",
    )
    verify_request_count(test_id, "PATCH", "/forms/id", None, 1)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .phone_attribute_identifier import PhoneAttributeIdentifier
from .signup_verified import SignupVerified


class PhoneAttribute(UniversalBaseModel):
    """
    Configuration for the phone number attribute for users.
    """

    identifier: typing.Optional[PhoneAttributeIdentifier] = None
    profile_required: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Determines if property should be required for users
    """

    signup: typing.Optional[SignupVerified] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .flow_action_stripe_add_tax_id import FlowActionStripeAddTaxId
from .flow_action_stripe_create_customer import FlowActionStripeCreateCustomer
from .flow_action_stripe_create_portal_session import FlowActionStripeCreatePortalSession
from .flow_action_stripe_delete_tax_id import FlowActionStripeDeleteTaxId
from .flow_action_stripe_find_customers import FlowActionStripeFindCustomers
from .flow_action_stripe_get_customer import FlowActionStripeGetCustomer
from .flow_action_stripe_update_customer import FlowActionStripeUpdateCustomer

FlowActionStripe = typing.Union[
    FlowActionStripeAddTaxId,
    FlowActionStripeCreateCustomer,
    FlowActionStripeCreatePortalSession,
    FlowActionStripeDeleteTaxId,
    FlowActionStripeFindCustomers,
    FlowActionStripeGetCustomer,
    FlowActionStripeUpdateCustomer,
]



# This file was auto-generated by Fern from our API Definition.

import typing

ConnectionResponseTypesSupported = typing.List[str]
"""
A list of the OAuth 2.0 response_type values that this OP supports. Dynamic OpenID Providers MUST support the code, id_token, and the token id_token Response Type values
"""



# This file was auto-generated by Fern from our API Definition.

import typing

from .flow_action_activecampaign_list_contacts import FlowActionActivecampaignListContacts
from .flow_action_activecampaign_upsert_contact import FlowActionActivecampaignUpsertContact

FlowActionActivecampaign = typing.Union[FlowActionActivecampaignListContacts, FlowActionActivecampaignUpsertContact]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .raw_client import AsyncRawVaultClient, RawVaultClient

if typing.TYPE_CHECKING:
    from .connections.client import AsyncConnectionsClient, ConnectionsClient


class VaultClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawVaultClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._connections: typing.Optional[ConnectionsClient] = None

    @property
    def with_raw_response(self) -> RawVaultClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawVaultClient
        """
        return self._raw_client

    @property
    def connections(self):
        if self._connections is None:
            from .connections.client import ConnectionsClient  # noqa: E402

            self._connections = ConnectionsClient(client_wrapper=self._client_wrapper)
        return self._connections


class AsyncVaultClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawVaultClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._connections: typing.Optional[AsyncConnectionsClient] = None

    @property
    def with_raw_response(self) -> AsyncRawVaultClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawVaultClient
        """
        return self._raw_client

    @property
    def connections(self):
        if self._connections is None:
            from .connections.client import AsyncConnectionsClient  # noqa: E402

            self._connections = AsyncConnectionsClient(client_wrapper=self._client_wrapper)
        return self._connections



from typing import Any, Optional, Union, List, Dict

from .base import AuthenticationBase

import json


class BackChannelLogin(AuthenticationBase):
    """Back-Channel Login endpoint"""

    def back_channel_login(
        self,
        binding_message: str,
        login_hint: str,
        scope: str,
        authorization_details: Optional[Union[str, List[Dict]]] = None,
        requested_expiry: Optional[int] = None,
        **kwargs
    ) -> Any:
        """Send a Back-Channel Login.

        Args:
             binding_message (str): Human-readable string displayed on both the device calling /bc-authorize and the user’s 
             authentication device to ensure the user is approves the correct request.

             login_hint (str):  JSON string containing user details for authentication in the iss_sub format.Ensure 
             serialization before passing.

             scope(str): "openid" is a required scope.Multiple scopes are separated 
             with whitespace.

             authorization_details (str, list of dict, optional): JSON string or a list of dictionaries representing
             Rich Authorization Requests (RAR) details to include in the CIBA request.

             requested_expiry (int, optional): Number of seconds the authentication request is valid for.
                Auth0 defaults to 300 seconds (5 mins) if not provided.

             **kwargs: Other fields to send along with the request.

        Returns:
            auth_req_id, expires_in, interval
        """

        data = {
                "client_id": self.client_id,
                "binding_message": binding_message,
                "login_hint": login_hint,
                "scope": scope,
                **kwargs,
            }

        if authorization_details is not None:
            if isinstance(authorization_details, str):
                data["authorization_details"] = authorization_details
            elif isinstance(authorization_details, list):
                data["authorization_details"] = json.dumps(authorization_details)

        if requested_expiry is not None:
            if not isinstance(requested_expiry, int) or requested_expiry <= 0:
                raise ValueError("requested_expiry must be a positive integer")
            data["requested_expiry"] = str(requested_expiry)

        data.update(kwargs)

        return self.authenticated_post(
            f"{self.protocol}://{self.domain}/bc-authorize",
            data = data,
            headers={"Content-Type": "application/x-www-form-urlencoded"},
        )



# This file was auto-generated by Fern from our API Definition.

import typing

EventStreamCloudEventUserCreatedObjectIdentitiesItemDatabaseProviderEnum = typing.Union[
    typing.Literal["auth0"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

from .connection_tenant_domain import ConnectionTenantDomain

ConnectionTenantDomainGoogleApps = ConnectionTenantDomain
"""
The Google Workspace primary domain used to identify the organization during authentication.
"""



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .tenant_settings_country_codes_mode_response import TenantSettingsCountryCodesModeResponse


class TenantSettingsCountryCodesResponse(UniversalBaseModel):
    """
    Phone country code configuration for identifier input.
    """

    list_: typing_extensions.Annotated[
        typing.Optional[typing.List[str]],
        FieldMetadata(alias="list"),
        pydantic.Field(alias="list", description="Array of ISO 3166-1 alpha-2 country codes."),
    ] = None
    """
    Array of ISO 3166-1 alpha-2 country codes.
    """

    mode: typing.Optional[TenantSettingsCountryCodesModeResponse] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



from .conftest import get_client, verify_request_count

from auth0.management import (
    BrandingThemeBorders,
    BrandingThemeColors,
    BrandingThemeFontBodyText,
    BrandingThemeFontButtonsText,
    BrandingThemeFontInputLabels,
    BrandingThemeFontLinks,
    BrandingThemeFonts,
    BrandingThemeFontSubtitle,
    BrandingThemeFontTitle,
    BrandingThemePageBackground,
    BrandingThemeWidget,
)


def test_branding_themes_create() -> None:
    """Test create endpoint with WireMock"""
    test_id = "branding.themes.create.0"
    client = get_client(test_id)
    client.branding.themes.create(
        borders=BrandingThemeBorders(
            button_border_radius=1.1,
            button_border_weight=1.1,
            buttons_style="pill",
            input_border_radius=1.1,
            input_border_weight=1.1,
            inputs_style="pill",
            show_widget_shadow=True,
            widget_border_weight=1.1,
            widget_corner_radius=1.1,
        ),
        colors=BrandingThemeColors(
            body_text="body_text",
            error="error",
            header="header",
            icons="icons",
            input_background="input_background",
            input_border="input_border",
            input_filled_text="input_filled_text",
            input_labels_placeholders="input_labels_placeholders",
            links_focused_components="links_focused_components",
            primary_button="primary_button",
            primary_button_label="primary_button_label",
            secondary_button_border="secondary_button_border",
            secondary_button_label="secondary_button_label",
            success="success",
            widget_background="widget_background",
            widget_border="widget_border",
        ),
        fonts=BrandingThemeFonts(
            body_text=BrandingThemeFontBodyText(
                bold=True,
                size=1.1,
            ),
            buttons_text=BrandingThemeFontButtonsText(
                bold=True,
                size=1.1,
            ),
            font_url="font_url",
            input_labels=BrandingThemeFontInputLabels(
                bold=True,
                size=1.1,
            ),
            links=BrandingThemeFontLinks(
                bold=True,
                size=1.1,
            ),
            links_style="normal",
            reference_text_size=1.1,
            subtitle=BrandingThemeFontSubtitle(
                bold=True,
                size=1.1,
            ),
            title=BrandingThemeFontTitle(
                bold=True,
                size=1.1,
            ),
        ),
        page_background=BrandingThemePageBackground(
            background_color="background_color",
            background_image_url="background_image_url",
            page_layout="center",
        ),
        widget=BrandingThemeWidget(
            header_text_alignment="center",
            logo_height=1.1,
            logo_position="center",
            logo_url="logo_url",
            social_buttons_layout="bottom",
        ),
    )
    verify_request_count(test_id, "POST", "/branding/themes", None, 1)


def test_branding_themes_get_default() -> None:
    """Test getDefault endpoint with WireMock"""
    test_id = "branding.themes.get_default.0"
    client = get_client(test_id)
    client.branding.themes.get_default()
    verify_request_count(test_id, "GET", "/branding/themes/default", None, 1)


def test_branding_themes_get() -> None:
    """Test get endpoint with WireMock"""
    test_id = "branding.themes.get.0"
    client = get_client(test_id)
    client.branding.themes.get(
        theme_id="themeId",
    )
    verify_request_count(test_id, "GET", "/branding/themes/themeId", None, 1)


def test_branding_themes_delete() -> None:
    """Test delete endpoint with WireMock"""
    test_id = "branding.themes.delete.0"
    client = get_client(test_id)
    client.branding.themes.delete(
        theme_id="themeId",
    )
    verify_request_count(test_id, "DELETE", "/branding/themes/themeId", None, 1)


def test_branding_themes_update() -> None:
    """Test update endpoint with WireMock"""
    test_id = "branding.themes.update.0"
    client = get_client(test_id)
    client.branding.themes.update(
        theme_id="themeId",
        borders=BrandingThemeBorders(
            button_border_radius=1.1,
            button_border_weight=1.1,
            buttons_style="pill",
            input_border_radius=1.1,
            input_border_weight=1.1,
            inputs_style="pill",
            show_widget_shadow=True,
            widget_border_weight=1.1,
            widget_corner_radius=1.1,
        ),
        colors=BrandingThemeColors(
            body_text="body_text",
            error="error",
            header="header",
            icons="icons",
            input_background="input_background",
            input_border="input_border",
            input_filled_text="input_filled_text",
            input_labels_placeholders="input_labels_placeholders",
            links_focused_components="links_focused_components",
            primary_button="primary_button",
            primary_button_label="primary_button_label",
            secondary_button_border="secondary_button_border",
            secondary_button_label="secondary_button_label",
            success="success",
            widget_background="widget_background",
            widget_border="widget_border",
        ),
        fonts=BrandingThemeFonts(
            body_text=BrandingThemeFontBodyText(
                bold=True,
                size=1.1,
            ),
            buttons_text=BrandingThemeFontButtonsText(
                bold=True,
                size=1.1,
            ),
            font_url="font_url",
            input_labels=BrandingThemeFontInputLabels(
                bold=True,
                size=1.1,
            ),
            links=BrandingThemeFontLinks(
                bold=True,
                size=1.1,
            ),
            links_style="normal",
            reference_text_size=1.1,
            subtitle=BrandingThemeFontSubtitle(
                bold=True,
                size=1.1,
            ),
            title=BrandingThemeFontTitle(
                bold=True,
                size=1.1,
            ),
        ),
        page_background=BrandingThemePageBackground(
            background_color="background_color",
            background_image_url="background_image_url",
            page_layout="center",
        ),
        widget=BrandingThemeWidget(
            header_text_alignment="center",
            logo_height=1.1,
            logo_position="center",
            logo_url="logo_url",
            social_buttons_layout="bottom",
        ),
    )
    verify_request_count(test_id, "PATCH", "/branding/themes/themeId", None, 1)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class EventStreamCloudEvent(UniversalBaseModel):
    """
    Event content. This will only be set if delivery failed.
    """

    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Unique identifier for the event
    """

    source: typing.Optional[str] = pydantic.Field(default=None)
    """
    Where the event originated
    """

    specversion: typing.Optional[str] = pydantic.Field(default=None)
    """
    Version of CloudEvents spec
    """

    type: typing.Optional[str] = pydantic.Field(default=None)
    """
    Type of the event (e.g., user.created)
    """

    time: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    Timestamp at which the event was generated
    """

    data: typing.Optional[str] = pydantic.Field(default=None)
    """
    Event contents encoded as a string.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import encode_path_param
from ..core.parse_error import ParsingError
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..errors.bad_request_error import BadRequestError
from ..errors.forbidden_error import ForbiddenError
from ..errors.not_found_error import NotFoundError
from ..errors.too_many_requests_error import TooManyRequestsError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.get_session_response_content import GetSessionResponseContent
from ..types.session_metadata import SessionMetadata
from ..types.update_session_response_content import UpdateSessionResponseContent
from pydantic import ValidationError

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawSessionsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetSessionResponseContent]:
        """
        Retrieve session information.

        Parameters
        ----------
        id : str
            ID of session to retrieve

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetSessionResponseContent]
            The session was retrieved
        """
        _response = self._client_wrapper.httpx_client.request(
            f"sessions/{encode_path_param(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetSessionResponseContent,
                    parse_obj_as(
                        type_=GetSessionResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> HttpResponse[None]:
        """
        Delete a session by ID.

        Parameters
        ----------
        id : str
            ID of the session to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"sessions/{encode_path_param(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update(
        self,
        id: str,
        *,
        session_metadata: typing.Optional[SessionMetadata] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[UpdateSessionResponseContent]:
        """
        Update session information.

        Parameters
        ----------
        id : str
            ID of the session to update.

        session_metadata : typing.Optional[SessionMetadata]
            Metadata associated with the session. Pass null or {} to remove all session_metadata.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpdateSessionResponseContent]
            Session successfully updated.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"sessions/{encode_path_param(id)}",
            method="PATCH",
            json={
                "session_metadata": session_metadata,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateSessionResponseContent,
                    parse_obj_as(
                        type_=UpdateSessionResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def revoke(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> HttpResponse[None]:
        """
        Revokes a session by ID and all associated refresh tokens.

        Parameters
        ----------
        id : str
            ID of the session to revoke.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"sessions/{encode_path_param(id)}/revoke",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawSessionsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetSessionResponseContent]:
        """
        Retrieve session information.

        Parameters
        ----------
        id : str
            ID of session to retrieve

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetSessionResponseContent]
            The session was retrieved
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"sessions/{encode_path_param(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetSessionResponseContent,
                    parse_obj_as(
                        type_=GetSessionResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[None]:
        """
        Delete a session by ID.

        Parameters
        ----------
        id : str
            ID of the session to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"sessions/{encode_path_param(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update(
        self,
        id: str,
        *,
        session_metadata: typing.Optional[SessionMetadata] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[UpdateSessionResponseContent]:
        """
        Update session information.

        Parameters
        ----------
        id : str
            ID of the session to update.

        session_metadata : typing.Optional[SessionMetadata]
            Metadata associated with the session. Pass null or {} to remove all session_metadata.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpdateSessionResponseContent]
            Session successfully updated.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"sessions/{encode_path_param(id)}",
            method="PATCH",
            json={
                "session_metadata": session_metadata,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateSessionResponseContent,
                    parse_obj_as(
                        type_=UpdateSessionResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def revoke(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[None]:
        """
        Revokes a session by ID and all associated refresh tokens.

        Parameters
        ----------
        id : str
            ID of the session to revoke.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"sessions/{encode_path_param(id)}/revoke",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.http_response import AsyncHttpResponse, HttpResponse
from ...core.parse_error import ParsingError
from ...core.pydantic_utilities import parse_obj_as
from ...core.request_options import RequestOptions
from ...core.serialization import convert_and_respect_annotation_metadata
from ...errors.bad_request_error import BadRequestError
from ...errors.conflict_error import ConflictError
from ...errors.forbidden_error import ForbiddenError
from ...errors.not_found_error import NotFoundError
from ...errors.payment_required_error import PaymentRequiredError
from ...errors.too_many_requests_error import TooManyRequestsError
from ...errors.unauthorized_error import UnauthorizedError
from ...types.get_universal_login_template_response_content import GetUniversalLoginTemplateResponseContent
from ...types.update_universal_login_template_request_content import UpdateUniversalLoginTemplateRequestContent
from pydantic import ValidationError

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawTemplatesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_universal_login(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetUniversalLoginTemplateResponseContent]:
        """
        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetUniversalLoginTemplateResponseContent]
            Template successfully retrieved.
        """
        _response = self._client_wrapper.httpx_client.request(
            "branding/templates/universal-login",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetUniversalLoginTemplateResponseContent,
                    parse_obj_as(
                        type_=GetUniversalLoginTemplateResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 402:
                raise PaymentRequiredError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update_universal_login(
        self,
        *,
        request: UpdateUniversalLoginTemplateRequestContent,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[None]:
        """
        Update the Universal Login branding template.

        When `content-type` header is set to `application/json`:

        ```json
        {
          "template": "<!DOCTYPE html>{% assign resolved_dir = dir | default: \\"auto\\" %}<html lang=\\"{{locale}}\\" dir=\\"{{resolved_dir}}\\"><head>{%- auth0:head -%}</head><body class=\\"_widget-auto-layout\\">{%- auth0:widget -%}</body></html>"
        }
        ```

        When `content-type` header is set to `text/html`:

        ```html
        <!DOCTYPE html>
        {% assign resolved_dir = dir | default: "auto" %}
        <html lang="{{locale}}" dir="{{resolved_dir}}">
          <head>
            {%- auth0:head -%}
          </head>
          <body class="_widget-auto-layout">
            {%- auth0:widget -%}
          </body>
        </html>
        ```

        Parameters
        ----------
        request : UpdateUniversalLoginTemplateRequestContent

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            "branding/templates/universal-login",
            method="PUT",
            json=convert_and_respect_annotation_metadata(
                object_=request, annotation=UpdateUniversalLoginTemplateRequestContent, direction="write"
            ),
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 402:
                raise PaymentRequiredError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete_universal_login(self, *, request_options: typing.Optional[RequestOptions] = None) -> HttpResponse[None]:
        """
        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            "branding/templates/universal-login",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 402:
                raise PaymentRequiredError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawTemplatesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_universal_login(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetUniversalLoginTemplateResponseContent]:
        """
        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetUniversalLoginTemplateResponseContent]
            Template successfully retrieved.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "branding/templates/universal-login",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetUniversalLoginTemplateResponseContent,
                    parse_obj_as(
                        type_=GetUniversalLoginTemplateResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 402:
                raise PaymentRequiredError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update_universal_login(
        self,
        *,
        request: UpdateUniversalLoginTemplateRequestContent,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[None]:
        """
        Update the Universal Login branding template.

        When `content-type` header is set to `application/json`:

        ```json
        {
          "template": "<!DOCTYPE html>{% assign resolved_dir = dir | default: \\"auto\\" %}<html lang=\\"{{locale}}\\" dir=\\"{{resolved_dir}}\\"><head>{%- auth0:head -%}</head><body class=\\"_widget-auto-layout\\">{%- auth0:widget -%}</body></html>"
        }
        ```

        When `content-type` header is set to `text/html`:

        ```html
        <!DOCTYPE html>
        {% assign resolved_dir = dir | default: "auto" %}
        <html lang="{{locale}}" dir="{{resolved_dir}}">
          <head>
            {%- auth0:head -%}
          </head>
          <body class="_widget-auto-layout">
            {%- auth0:widget -%}
          </body>
        </html>
        ```

        Parameters
        ----------
        request : UpdateUniversalLoginTemplateRequestContent

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "branding/templates/universal-login",
            method="PUT",
            json=convert_and_respect_annotation_metadata(
                object_=request, annotation=UpdateUniversalLoginTemplateRequestContent, direction="write"
            ),
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 402:
                raise PaymentRequiredError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete_universal_login(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[None]:
        """
        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "branding/templates/universal-login",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 402:
                raise PaymentRequiredError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)
